"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExecutionResponseType = void 0;
exports.createBulkExecutionEnqueuerFunction = createBulkExecutionEnqueuerFunction;
var _saved_objects = require("./constants/saved_objects");
var _lib = require("./lib");
var _has_reached_queued_actions_limit = require("./lib/has_reached_queued_actions_limit");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */
let ExecutionResponseType = exports.ExecutionResponseType = /*#__PURE__*/function (ExecutionResponseType) {
  ExecutionResponseType["SUCCESS"] = "success";
  ExecutionResponseType["QUEUED_ACTIONS_LIMIT_ERROR"] = "queuedActionsLimitError";
  return ExecutionResponseType;
}({});
function createBulkExecutionEnqueuerFunction({
  taskManager,
  actionTypeRegistry,
  isESOCanEncrypt,
  inMemoryConnectors,
  configurationUtilities,
  logger
}) {
  return async function execute(unsecuredSavedObjectsClient, actionsToExecute) {
    if (!isESOCanEncrypt) {
      throw new Error(`Unable to execute actions because the Encrypted Saved Objects plugin is missing encryption key. Please set xpack.encryptedSavedObjects.encryptionKey in the kibana.yml or use the bin/kibana-encryption-keys command.`);
    }
    const {
      hasReachedLimit,
      numberOverLimit
    } = await (0, _has_reached_queued_actions_limit.hasReachedTheQueuedActionsLimit)(taskManager, configurationUtilities, actionsToExecute.length);
    let actionsOverLimit = [];
    if (hasReachedLimit) {
      actionsOverLimit = actionsToExecute.splice(actionsToExecute.length - numberOverLimit, numberOverLimit);
    }
    const actionTypeIds = {};
    const spaceIds = {};
    const connectorIsInMemory = {};
    const connectorIds = [...new Set(actionsToExecute.map(action => action.id))];
    const connectors = await getConnectors(unsecuredSavedObjectsClient, inMemoryConnectors, connectorIds);
    const runnableActions = [];
    for (const c of connectors) {
      const {
        id,
        connector,
        isInMemory
      } = c;
      const {
        actionTypeId,
        name
      } = connector;
      try {
        validateConnector({
          id,
          connector,
          actionTypeRegistry
        });
      } catch (e) {
        logger.warn(`Skipped the actions for the connector: ${name} (${id}). Error: ${e.message}`);
        continue;
      }
      const actionsWithCurrentConnector = actionsToExecute.filter(action => action.id === id);
      if (actionsWithCurrentConnector.length > 0) {
        runnableActions.push(...actionsWithCurrentConnector);
      }
      actionTypeIds[id] = actionTypeId;
      connectorIsInMemory[id] = isInMemory;
    }
    const actions = runnableActions.map(actionToExecute => {
      // Get saved object references from action ID and relatedSavedObjects
      const {
        references,
        relatedSavedObjectWithRefs
      } = (0, _lib.extractSavedObjectReferences)(actionToExecute.id, connectorIsInMemory[actionToExecute.id], actionToExecute.relatedSavedObjects);
      const executionSourceReference = executionSourceAsSavedObjectReferences(actionToExecute.source);
      const taskReferences = [];
      if (executionSourceReference.references) {
        taskReferences.push(...executionSourceReference.references);
      }
      if (references) {
        taskReferences.push(...references);
      }
      spaceIds[actionToExecute.id] = actionToExecute.spaceId;
      return {
        type: _saved_objects.ACTION_TASK_PARAMS_SAVED_OBJECT_TYPE,
        attributes: {
          actionId: actionToExecute.id,
          params: actionToExecute.params,
          apiKey: actionToExecute.apiKey,
          executionId: actionToExecute.executionId,
          consumer: actionToExecute.consumer,
          relatedSavedObjects: relatedSavedObjectWithRefs,
          apiKeyId: actionToExecute.apiKeyId,
          ...(actionToExecute.source ? {
            source: actionToExecute.source.type
          } : {})
        },
        references: taskReferences
      };
    });
    const actionTaskParamsRecords = await unsecuredSavedObjectsClient.bulkCreate(actions, {
      refresh: false
    });
    const taskInstances = actionTaskParamsRecords.saved_objects.map((so, index) => {
      var _runnableActions$inde;
      const actionId = so.attributes.actionId;
      return {
        taskType: `actions:${actionTypeIds[actionId]}`,
        params: {
          spaceId: spaceIds[actionId],
          actionTaskParamsId: so.id
        },
        state: {},
        scope: ['actions'],
        ...((_runnableActions$inde = runnableActions[index]) !== null && _runnableActions$inde !== void 0 && _runnableActions$inde.priority ? {
          priority: runnableActions[index].priority
        } : {})
      };
    });
    await taskManager.bulkSchedule(taskInstances);
    return {
      errors: actionsOverLimit.length > 0,
      items: runnableActions.map(a => ({
        id: a.id,
        uuid: a.uuid,
        actionTypeId: a.actionTypeId,
        response: ExecutionResponseType.SUCCESS
      })).concat(actionsOverLimit.map(a => ({
        id: a.id,
        uuid: a.uuid,
        actionTypeId: a.actionTypeId,
        response: ExecutionResponseType.QUEUED_ACTIONS_LIMIT_ERROR
      })))
    };
  };
}
function validateConnector({
  id,
  connector,
  actionTypeRegistry
}) {
  const {
    name,
    isMissingSecrets,
    actionTypeId
  } = connector;
  if (isMissingSecrets) {
    throw new Error(`Unable to execute action because no secrets are defined for the "${name}" connector.`);
  }
  if (!actionTypeRegistry.isActionExecutable(id, actionTypeId, {
    notifyUsage: true
  })) {
    actionTypeRegistry.ensureActionTypeEnabled(actionTypeId);
  }
}
function executionSourceAsSavedObjectReferences(executionSource) {
  return (0, _lib.isSavedObjectExecutionSource)(executionSource) ? {
    references: [{
      name: 'source',
      ...executionSource.source
    }]
  } : {};
}
async function getConnectors(unsecuredSavedObjectsClient, inMemoryConnectors, connectorIds) {
  const result = [];
  const connectorIdsToFetch = [];
  for (const connectorId of connectorIds) {
    const pcConnector = inMemoryConnectors.find(connector => connector.id === connectorId);
    if (pcConnector) {
      result.push({
        connector: pcConnector,
        isInMemory: true,
        id: connectorId
      });
    } else {
      connectorIdsToFetch.push(connectorId);
    }
  }
  if (connectorIdsToFetch.length > 0) {
    const bulkGetResult = await unsecuredSavedObjectsClient.bulkGet(connectorIdsToFetch.map(id => ({
      id,
      type: 'action'
    })));
    for (const item of bulkGetResult.saved_objects) {
      if (item.error) throw item.error;
      result.push({
        isInMemory: false,
        connector: item.attributes,
        id: item.id
      });
    }
  }
  return result;
}