"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.upgradePackagePolicyHandler = exports.updatePackagePolicyHandler = exports.isNotNull = exports.getPackagePoliciesHandler = exports.getOrphanedPackagePolicies = exports.getOnePackagePolicyHandler = exports.dryRunUpgradePackagePolicyHandler = exports.deletePackagePolicyHandler = exports.deleteOnePackagePolicyHandler = exports.createPackagePolicyHandler = exports.bulkGetPackagePoliciesHandler = void 0;
var _server = require("@kbn/core/server");
var _lodash = require("lodash");
var _http_authorization_header = require("../../../common/http_authorization_header");
var _populate_package_policy_assigned_agents_count = require("../../services/package_policies/populate_package_policy_assigned_agents_count");
var _services = require("../../services");
var _constants = require("../../../common/constants");
var _errors = require("../../errors");
var _packages = require("../../services/epm/packages");
var _constants2 = require("../../constants");
var _simplified_package_policy_helper = require("../../../common/services/simplified_package_policy_helper");
var _agentless_policy_helper = require("../../../common/services/agentless_policy_helper");
var _utils = require("./utils");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const isNotNull = value => value !== null;
exports.isNotNull = isNotNull;
const getPackagePoliciesHandler = async (context, request, response) => {
  const esClient = (await context.core).elasticsearch.client.asInternalUser;
  const fleetContext = await context.fleet;
  const soClient = fleetContext.internalSoClient;
  const limitedToPackages = fleetContext.limitedToPackages;
  const {
    items,
    total,
    page,
    perPage
  } = await _services.packagePolicyService.list(soClient, request.query);
  (0, _services.checkAllowedPackages)(items, limitedToPackages, 'package.name');
  if (request.query.withAgentCount) {
    await (0, _populate_package_policy_assigned_agents_count.populatePackagePolicyAssignedAgentsCount)(esClient, items);
  }

  // agnostic to package-level RBAC
  return response.ok({
    body: {
      items: request.query.format === _constants.inputsFormat.Simplified ? items.map(item => (0, _simplified_package_policy_helper.packagePolicyToSimplifiedPackagePolicy)(item)) : items,
      total,
      page,
      perPage
    }
  });
};
exports.getPackagePoliciesHandler = getPackagePoliciesHandler;
const bulkGetPackagePoliciesHandler = async (context, request, response) => {
  const fleetContext = await context.fleet;
  const soClient = fleetContext.internalSoClient;
  const limitedToPackages = fleetContext.limitedToPackages;
  const {
    ids,
    ignoreMissing
  } = request.body;
  try {
    const items = await _services.packagePolicyService.getByIDs(soClient, ids, {
      ignoreMissing
    });
    const responseItems = items !== null && items !== void 0 ? items : [];
    (0, _services.checkAllowedPackages)(responseItems, limitedToPackages, 'package.name');
    return response.ok({
      body: {
        items: responseItems.length > 0 && request.query.format === _constants.inputsFormat.Simplified ? responseItems.map(item => (0, _simplified_package_policy_helper.packagePolicyToSimplifiedPackagePolicy)(item)) : responseItems
      }
    });
  } catch (error) {
    if (error instanceof _errors.PackagePolicyNotFoundError) {
      return response.notFound({
        body: {
          message: error.message
        }
      });
    }
    throw error;
  }
};
exports.bulkGetPackagePoliciesHandler = bulkGetPackagePoliciesHandler;
const getOnePackagePolicyHandler = async (context, request, response) => {
  const fleetContext = await context.fleet;
  const soClient = fleetContext.internalSoClient;
  const limitedToPackages = fleetContext.limitedToPackages;
  const {
    packagePolicyId
  } = request.params;
  const notFoundResponse = () => response.notFound({
    body: {
      message: `Package policy ${packagePolicyId} not found`
    }
  });
  try {
    const packagePolicy = await _services.packagePolicyService.get(soClient, packagePolicyId);
    if (packagePolicy) {
      (0, _services.checkAllowedPackages)([packagePolicy], limitedToPackages, 'package.name');
      return response.ok({
        body: {
          item: request.query.format === _constants.inputsFormat.Simplified ? (0, _simplified_package_policy_helper.packagePolicyToSimplifiedPackagePolicy)(packagePolicy) : packagePolicy
        }
      });
    } else {
      return notFoundResponse();
    }
  } catch (error) {
    if (_server.SavedObjectsErrorHelpers.isNotFoundError(error)) {
      return notFoundResponse();
    }
    throw error;
  }
};
exports.getOnePackagePolicyHandler = getOnePackagePolicyHandler;
const getOrphanedPackagePolicies = async (context, request, response) => {
  const soClient = (await context.core).savedObjects.client;
  const installedPackages = await (0, _packages.getInstallations)(soClient, {
    perPage: _constants2.SO_SEARCH_LIMIT,
    filter: `
        ${_constants2.PACKAGES_SAVED_OBJECT_TYPE}.attributes.install_status:${_constants.installationStatuses.Installed}
    `
  });
  const orphanedPackagePolicies = [];
  const packagePolicies = await _services.packagePolicyService.list(soClient, {
    perPage: _constants2.SO_SEARCH_LIMIT
  });
  const packagePoliciesByPackage = (0, _lodash.groupBy)(packagePolicies.items, 'package.name');
  const agentPolicies = await _services.agentPolicyService.list(soClient, {
    perPage: _constants2.SO_SEARCH_LIMIT
  });
  const agentPoliciesById = (0, _lodash.keyBy)(agentPolicies.items, 'id');
  const usedPackages = installedPackages.saved_objects.filter(({
    attributes: {
      name
    }
  }) => !!packagePoliciesByPackage[name]);
  usedPackages.forEach(({
    attributes: {
      name
    }
  }) => {
    packagePoliciesByPackage[name].forEach(packagePolicy => {
      if (packagePolicy.policy_ids.every(policyId => !agentPoliciesById[policyId])) {
        orphanedPackagePolicies.push(packagePolicy);
      }
    });
  });
  return response.ok({
    body: {
      items: orphanedPackagePolicies,
      total: orphanedPackagePolicies.length
    }
  });
};
exports.getOrphanedPackagePolicies = getOrphanedPackagePolicies;
const createPackagePolicyHandler = async (context, request, response) => {
  const coreContext = await context.core;
  const fleetContext = await context.fleet;
  const soClient = fleetContext.internalSoClient;
  const esClient = coreContext.elasticsearch.client.asInternalUser;
  const user = _services.appContextService.getSecurityCore().authc.getCurrentUser(request) || undefined;
  const {
    force,
    id,
    package: pkg,
    ...newPolicy
  } = request.body;
  const authorizationHeader = _http_authorization_header.HTTPAuthorizationHeader.parseFromRequest(request, user === null || user === void 0 ? void 0 : user.username);
  let wasPackageAlreadyInstalled = false;
  const spaceId = fleetContext.spaceId;
  try {
    let newPackagePolicy;
    if ((0, _utils.isSimplifiedCreatePackagePolicyRequest)(newPolicy)) {
      if (!pkg) {
        throw new _errors.PackagePolicyRequestError('Package is required');
      }
      const pkgInfo = await (0, _packages.getPackageInfo)({
        savedObjectsClient: soClient,
        pkgName: pkg.name,
        pkgVersion: pkg.version,
        ignoreUnverified: force,
        prerelease: true
      });
      newPackagePolicy = (0, _simplified_package_policy_helper.simplifiedPackagePolicytoNewPackagePolicy)(newPolicy, pkgInfo, {
        experimental_data_stream_features: pkg.experimental_data_stream_features
      });
    } else {
      newPackagePolicy = await _services.packagePolicyService.enrichPolicyWithDefaultsFromPackage(soClient, {
        ...newPolicy,
        package: pkg
      });
    }
    newPackagePolicy.inputs = (0, _utils.alignInputsAndStreams)(newPackagePolicy.inputs);
    const installation = await (0, _packages.getInstallation)({
      savedObjectsClient: soClient,
      pkgName: pkg.name
    });
    wasPackageAlreadyInstalled = (installation === null || installation === void 0 ? void 0 : installation.install_status) === 'installed';

    // Create package policy
    const packagePolicy = await fleetContext.packagePolicyService.asCurrentUser.create(soClient, esClient, newPackagePolicy, {
      id,
      force,
      spaceId,
      authorizationHeader
    }, context, request);
    return response.ok({
      body: {
        item: request.query.format === _constants.inputsFormat.Simplified ? (0, _simplified_package_policy_helper.packagePolicyToSimplifiedPackagePolicy)(packagePolicy) : packagePolicy
      }
    });
  } catch (error) {
    _services.appContextService.getLogger().error(`Error while creating package policy due to error: ${error.message}`);
    if (!wasPackageAlreadyInstalled) {
      const installation = await (0, _packages.getInstallation)({
        savedObjectsClient: soClient,
        pkgName: pkg.name
      });
      if (installation) {
        _services.appContextService.getLogger().info(`rollback ${pkg.name}-${pkg.version} package installation after error`);
        await (0, _packages.removeInstallation)({
          savedObjectsClient: soClient,
          pkgName: pkg.name,
          pkgVersion: pkg.version,
          esClient
        });
      }
    }
    if (error.statusCode) {
      return response.customError({
        statusCode: error.statusCode,
        body: {
          message: error.message
        }
      });
    }
    throw error;
  }
};
exports.createPackagePolicyHandler = createPackagePolicyHandler;
const updatePackagePolicyHandler = async (context, request, response) => {
  const coreContext = await context.core;
  const fleetContext = await context.fleet;
  const soClient = fleetContext.internalSoClient;
  const limitedToPackages = fleetContext.limitedToPackages;
  const esClient = coreContext.elasticsearch.client.asInternalUser;
  const user = _services.appContextService.getSecurityCore().authc.getCurrentUser(request) || undefined;
  const packagePolicy = await _services.packagePolicyService.get(soClient, request.params.packagePolicyId);
  if (!packagePolicy) {
    throw new _errors.PackagePolicyNotFoundError('Package policy not found');
  }
  if (limitedToPackages && limitedToPackages.length) {
    var _packagePolicy$packag;
    const packageName = packagePolicy === null || packagePolicy === void 0 ? void 0 : (_packagePolicy$packag = packagePolicy.package) === null || _packagePolicy$packag === void 0 ? void 0 : _packagePolicy$packag.name;
    if (packageName && !limitedToPackages.includes(packageName)) {
      return response.forbidden({
        body: {
          message: `Update for package name ${packageName} is not authorized.`
        }
      });
    }
  }
  try {
    var _packagePolicy$packag2;
    // simplified request
    const {
      force,
      package: pkg,
      ...body
    } = request.body;
    let newData;
    if (body.inputs && (0, _utils.isSimplifiedCreatePackagePolicyRequest)(body)) {
      if (!pkg) {
        throw new _errors.PackagePolicyRequestError('Package is required');
      }
      const pkgInfo = await (0, _packages.getPackageInfo)({
        savedObjectsClient: soClient,
        pkgName: pkg.name,
        pkgVersion: pkg.version
      });
      newData = (0, _simplified_package_policy_helper.simplifiedPackagePolicytoNewPackagePolicy)(body, pkgInfo, {
        experimental_data_stream_features: pkg.experimental_data_stream_features
      });
      (0, _agentless_policy_helper.validateAgentlessInputs)(body === null || body === void 0 ? void 0 : body.inputs, (body === null || body === void 0 ? void 0 : body.supports_agentless) || newData.supports_agentless);
    } else {
      var _restOfBody$name, _restOfBody$descripti, _restOfBody$namespace, _restOfBody$enabled, _restOfBody$inputs, _restOfBody$vars, _restOfBody$supports_;
      // complete request
      const {
        overrides,
        ...restOfBody
      } = body;
      const packagePolicyInputs = (0, _utils.removeFieldsFromInputSchema)(packagePolicy.inputs);

      // listing down accepted properties, because loaded packagePolicy contains some that are not accepted in update
      newData = {
        ...restOfBody,
        name: (_restOfBody$name = restOfBody.name) !== null && _restOfBody$name !== void 0 ? _restOfBody$name : packagePolicy.name,
        description: (_restOfBody$descripti = restOfBody.description) !== null && _restOfBody$descripti !== void 0 ? _restOfBody$descripti : packagePolicy.description,
        namespace: (_restOfBody$namespace = restOfBody.namespace) !== null && _restOfBody$namespace !== void 0 ? _restOfBody$namespace : packagePolicy === null || packagePolicy === void 0 ? void 0 : packagePolicy.namespace,
        policy_id: restOfBody.policy_id === undefined ? packagePolicy.policy_id : restOfBody.policy_id,
        enabled: 'enabled' in restOfBody ? (_restOfBody$enabled = restOfBody.enabled) !== null && _restOfBody$enabled !== void 0 ? _restOfBody$enabled : packagePolicy.enabled : packagePolicy.enabled,
        package: pkg !== null && pkg !== void 0 ? pkg : packagePolicy.package,
        inputs: (_restOfBody$inputs = restOfBody.inputs) !== null && _restOfBody$inputs !== void 0 ? _restOfBody$inputs : packagePolicyInputs,
        vars: (_restOfBody$vars = restOfBody.vars) !== null && _restOfBody$vars !== void 0 ? _restOfBody$vars : packagePolicy.vars,
        supports_agentless: (_restOfBody$supports_ = restOfBody.supports_agentless) !== null && _restOfBody$supports_ !== void 0 ? _restOfBody$supports_ : packagePolicy.supports_agentless
      };
      if (overrides) {
        newData.overrides = overrides;
      }
    }
    (0, _agentless_policy_helper.validateAgentlessInputs)(newData.inputs, newData.supports_agentless || packagePolicy.supports_agentless);
    newData.inputs = (0, _utils.alignInputsAndStreams)(newData.inputs);
    if (newData.policy_ids && !(0, _lodash.isEmpty)(packagePolicy.policy_ids) && !(0, _lodash.isEqual)(newData.policy_ids, packagePolicy.policy_ids)) {
      const agentPolicy = await _services.agentPolicyService.get(soClient, packagePolicy.policy_ids[0]);
      if (agentPolicy !== null && agentPolicy !== void 0 && agentPolicy.supports_agentless) {
        throw new _errors.PackagePolicyRequestError('Cannot change agent policies of an agentless integration');
      }
    }
    await (0, _utils.renameAgentlessAgentPolicy)(soClient, esClient, packagePolicy, newData.name);
    const updatedPackagePolicy = await _services.packagePolicyService.update(soClient, esClient, request.params.packagePolicyId, newData, {
      user,
      force
    }, (_packagePolicy$packag2 = packagePolicy.package) === null || _packagePolicy$packag2 === void 0 ? void 0 : _packagePolicy$packag2.version);
    return response.ok({
      body: {
        item: request.query.format === _constants.inputsFormat.Simplified ? (0, _simplified_package_policy_helper.packagePolicyToSimplifiedPackagePolicy)(updatedPackagePolicy) : updatedPackagePolicy
      }
    });
  } catch (error) {
    if (error.statusCode) {
      return response.customError({
        statusCode: error.statusCode,
        body: {
          message: error.message
        }
      });
    }
    throw error;
  }
};
exports.updatePackagePolicyHandler = updatePackagePolicyHandler;
const deletePackagePolicyHandler = async (context, request, response) => {
  const coreContext = await context.core;
  const soClient = coreContext.savedObjects.client;
  const esClient = coreContext.elasticsearch.client.asInternalUser;
  const user = _services.appContextService.getSecurityCore().authc.getCurrentUser(request) || undefined;
  const body = await _services.packagePolicyService.delete(soClient, esClient, request.body.packagePolicyIds, {
    user,
    force: request.body.force,
    skipUnassignFromAgentPolicies: request.body.force
  }, context, request);
  return response.ok({
    body
  });
};
exports.deletePackagePolicyHandler = deletePackagePolicyHandler;
const deleteOnePackagePolicyHandler = async (context, request, response) => {
  const coreContext = await context.core;
  const soClient = coreContext.savedObjects.client;
  const esClient = coreContext.elasticsearch.client.asInternalUser;
  const user = _services.appContextService.getSecurityCore().authc.getCurrentUser(request) || undefined;
  const res = await _services.packagePolicyService.delete(soClient, esClient, [request.params.packagePolicyId], {
    user,
    force: request.query.force,
    skipUnassignFromAgentPolicies: request.query.force
  }, context, request);
  if (res[0] && res[0].success === false && res[0].statusCode !== 404 // ignore 404 to allow that call to be idempotent
  ) {
    var _res$0$statusCode;
    return response.customError({
      statusCode: (_res$0$statusCode = res[0].statusCode) !== null && _res$0$statusCode !== void 0 ? _res$0$statusCode : 500,
      body: res[0].body
    });
  }
  return response.ok({
    body: {
      id: request.params.packagePolicyId
    }
  });
};
exports.deleteOnePackagePolicyHandler = deleteOnePackagePolicyHandler;
const upgradePackagePolicyHandler = async (context, request, response) => {
  const coreContext = await context.core;
  const soClient = coreContext.savedObjects.client;
  const esClient = coreContext.elasticsearch.client.asInternalUser;
  const user = _services.appContextService.getSecurityCore().authc.getCurrentUser(request) || undefined;
  const body = await _services.packagePolicyService.upgrade(soClient, esClient, request.body.packagePolicyIds, {
    user
  });
  const firstFatalError = body.find(item => item.statusCode && item.statusCode !== 200);
  if (firstFatalError) {
    return response.customError({
      statusCode: firstFatalError.statusCode,
      body: {
        message: firstFatalError.body.message
      }
    });
  }
  return response.ok({
    body
  });
};
exports.upgradePackagePolicyHandler = upgradePackagePolicyHandler;
const dryRunUpgradePackagePolicyHandler = async (context, request, response) => {
  const soClient = (await context.core).savedObjects.client;
  const body = [];
  const {
    packagePolicyIds
  } = request.body;
  for (const id of packagePolicyIds) {
    const result = await _services.packagePolicyService.getUpgradeDryRunDiff(soClient, id);
    body.push(result);
  }
  const firstFatalError = body.find(item => item.statusCode && item.statusCode !== 200);
  if (firstFatalError) {
    return response.customError({
      statusCode: firstFatalError.statusCode,
      body: {
        message: firstFatalError.body.message
      }
    });
  }
  return response.ok({
    body
  });
};
exports.dryRunUpgradePackagePolicyHandler = dryRunUpgradePackagePolicyHandler;