"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.executeCreateAction = executeCreateAction;
exports.getVisualizationMap = exports.getDatasourceMap = void 0;
exports.isCreateActionCompatible = isCreateActionCompatible;
exports.setVisualizationMap = exports.setDatasourceMap = void 0;
var _common = require("@kbn/kibana-utils-plugin/common");
var _visualizationUtils = require("@kbn/visualization-utils");
var _public = require("@kbn/ui-actions-plugin/public");
var _esqlUtils = require("@kbn/esql-utils");
var _lens_suggestions_api = require("../../lens_suggestions_api");
var _id_generator = require("../../id_generator");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

// datasourceMap and visualizationMap setters/getters
const [getVisualizationMap, setVisualizationMap] = (0, _common.createGetterSetter)('VisualizationMap', false);
exports.setVisualizationMap = setVisualizationMap;
exports.getVisualizationMap = getVisualizationMap;
const [getDatasourceMap, setDatasourceMap] = (0, _common.createGetterSetter)('DatasourceMap', false);
exports.setDatasourceMap = setDatasourceMap;
exports.getDatasourceMap = getDatasourceMap;
async function isCreateActionCompatible(core) {
  return core.uiSettings.get(_esqlUtils.ENABLE_ESQL);
}
async function executeCreateAction({
  deps,
  core,
  api,
  editorFrameService
}) {
  var _suggestionsApi, _embeddable$onEdit;
  const getFallbackDataView = async () => {
    const indexName = await (0, _esqlUtils.getIndexForESQLQuery)({
      dataViews: deps.dataViews
    });
    if (!indexName) return null;
    const dataView = await (0, _esqlUtils.getESQLAdHocDataview)(`from ${indexName}`, deps.dataViews);
    return dataView;
  };
  const [isCompatibleAction, dataView] = await Promise.all([isCreateActionCompatible(core), getFallbackDataView()]);
  if (!isCompatibleAction || !dataView) {
    throw new _public.IncompatibleActionError();
  }
  let visualizationMap = getVisualizationMap();
  let datasourceMap = getDatasourceMap();
  if (!visualizationMap || !datasourceMap) {
    [visualizationMap, datasourceMap] = await Promise.all([editorFrameService.loadVisualizations(), editorFrameService.loadDatasources()]);
    if (!visualizationMap && !datasourceMap) {
      throw new _public.IncompatibleActionError();
    }

    // persist for retrieval elsewhere
    setDatasourceMap(datasourceMap);
    setVisualizationMap(visualizationMap);
  }
  const esqlQuery = (0, _esqlUtils.getInitialESQLQuery)(dataView);
  const defaultEsqlQuery = {
    esql: esqlQuery
  };

  // For the suggestions api we need only the columns
  // so we are requesting them with limit 0
  // this is much more performant than requesting
  // all the table
  const abortController = new AbortController();
  const columns = await (0, _esqlUtils.getESQLQueryColumns)({
    esqlQuery,
    search: deps.data.search.search,
    signal: abortController.signal,
    timeRange: deps.data.query.timefilter.timefilter.getAbsoluteTime()
  });
  const context = {
    dataViewSpec: dataView.toSpec(false),
    fieldName: '',
    textBasedColumns: columns,
    query: defaultEsqlQuery
  };

  // get the initial attributes from the suggestions api
  const allSuggestions = (_suggestionsApi = (0, _lens_suggestions_api.suggestionsApi)({
    context,
    dataView,
    datasourceMap,
    visualizationMap
  })) !== null && _suggestionsApi !== void 0 ? _suggestionsApi : [];

  // Lens might not return suggestions for some cases, i.e. in case of errors
  if (!allSuggestions.length) return undefined;
  const [firstSuggestion] = allSuggestions;
  const attrs = (0, _visualizationUtils.getLensAttributesFromSuggestion)({
    filters: [],
    query: defaultEsqlQuery,
    suggestion: {
      ...firstSuggestion,
      title: '' // when creating a new panel, we don't want to use the title from the suggestion
    },
    dataView
  });
  const embeddable = await api.addNewPanel({
    panelType: 'lens',
    initialState: {
      attributes: attrs,
      id: (0, _id_generator.generateId)(),
      isNewPanel: true
    }
  });
  // open the flyout if embeddable has been created successfully
  embeddable === null || embeddable === void 0 ? void 0 : (_embeddable$onEdit = embeddable.onEdit) === null || _embeddable$onEdit === void 0 ? void 0 : _embeddable$onEdit.call(embeddable);
}